@@ -63,6 +63,31 @@ class GenericMedia(object):
             return 'video'
 
 
+    def get_translated_type(self, upper_flag=False):
+
+        if not upper_flag:
+
+            if isinstance(self, Channel):
+                return _('channel')
+            elif isinstance(self, Playlist):
+                return _('playlist')
+            elif isinstance(self, Folder):
+                return _('folder')
+            else:
+                return _('video')
+
+        else:
+
+            if isinstance(self, Channel):
+                return _('Channel')
+            elif isinstance(self, Playlist):
+                return _('Playlist')
+            elif isinstance(self, Folder):
+                return _('Folder')
+            else:
+                return _('Video')
+
+
     # Set accessors
 
 
@@ -259,29 +284,48 @@ class GenericContainer(GenericMedia):
         else:
             self.child_list.remove(child_obj)
 
+            # Git #169, v2.2.026. A user reports that the counts can fall below
+            #   0. The authors can't reproduce the problem, but we can still
+            #   prevent negative values
             if isinstance(child_obj, Video):
                 self.vid_count -= 1
+                if self.vid_count < 0:
+                    self.vid_count = 0
 
                 if child_obj.bookmark_flag:
                     self.bookmark_count -= 1
+                    if self.bookmark_count < 0:
+                        self.bookmark_count = 0
 
                 if child_obj.dl_flag:
                     self.dl_count -= 1
+                    if self.dl_count < 0:
+                        self.dl_count = 0
 
                 if child_obj.fav_flag:
                     self.fav_count -= 1
+                    if self.fav_count < 0:
+                        self.fav_count = 0
 
                 if child_obj.live_mode:
                     self.live_count -= 1
+                    if self.live_count < 0:
+                        self.live_count = 0
 
                 if child_obj.missing_flag:
                     self.missing_count -= 1
+                    if self.missing_count < 0:
+                        self.missing_count = 0
 
                 if child_obj.new_flag:
                     self.new_count -= 1
+                    if self.new_count < 0:
+                        self.new_count = 0
 
                 if child_obj.waiting_flag:
                     self.waiting_count -= 1
+                    if self.waiting_count < 0:
+                        self.waiting_count = 0
 
             return True
 
@@ -719,6 +763,69 @@ class GenericContainer(GenericMedia):
                     self.waiting_count += 1
 
 
+    def test_counts(self):
+
+        """Can be called by anything.
+
+        Recalculates all count IVs, and compares them against the container's
+        actual counts.
+
+        Return values:
+
+            True if there is a discrepancy; False if the container's actual
+                counts seem to be correct
+
+        """
+
+        vid_count = 0
+        bookmark_count = 0
+        dl_count = 0
+        fav_count = 0
+        live_count = 0
+        missing_count = 0
+        new_count = 0
+        waiting_count = 0
+
+        for child_obj in self.child_list:
+
+            if isinstance(child_obj, Video):
+                vid_count += 1
+
+                if child_obj.bookmark_flag:
+                    bookmark_count += 1
+
+                if child_obj.dl_flag:
+                    dl_count += 1
+
+                if child_obj.fav_flag:
+                    fav_count += 1
+
+                if child_obj.live_mode:
+                    live_count += 1
+
+                if child_obj.missing_flag:
+                    missing_count += 1
+
+                if child_obj.new_flag:
+                    new_count += 1
+
+                if child_obj.waiting_flag:
+                    waiting_count += 1
+
+        if vid_count != self.vid_count \
+        or bookmark_count != self.bookmark_count \
+        or dl_count != self.dl_count \
+        or fav_count != self.fav_count \
+        or live_count != self.live_count \
+        or missing_count != self.missing_count \
+        or new_count != self.new_count \
+        or waiting_count != self.waiting_count:
+            return True
+
+        else:
+            return False
+
+
     # Set accessors
 
 
@@ -752,6 +859,8 @@ class GenericContainer(GenericMedia):
     def dec_bookmark_count(self):
 
         self.bookmark_count -= 1
+        if self.bookmark_count < 0:
+            self.bookmark_count = 0
 
 
     def inc_dl_count(self):
@@ -762,6 +871,8 @@ class GenericContainer(GenericMedia):
     def dec_dl_count(self):
 
         self.dl_count -= 1
+        if self.dl_count < 0:
+            self.dl_count = 0
 
 
     def set_dl_disable_flag(self, flag):
@@ -780,6 +891,8 @@ class GenericContainer(GenericMedia):
     def dec_fav_count(self):
 
         self.fav_count -= 1
+        if self.fav_count < 0:
+            self.fav_count = 0
 
 
     def inc_live_count(self):
@@ -790,6 +903,8 @@ class GenericContainer(GenericMedia):
     def dec_live_count(self):
 
         self.live_count -= 1
+        if self.live_count < 0:
+            self.live_count = 0
 
 
     def set_master_dbid(self, app_obj, dbid):
@@ -832,6 +947,8 @@ class GenericContainer(GenericMedia):
     def dec_missing_count(self):
 
         self.missing_count -= 1
+        if self.missing_count < 0:
+            self.missing_count = 0
 
 
     def inc_new_count(self):
@@ -842,6 +959,8 @@ class GenericContainer(GenericMedia):
     def dec_new_count(self):
 
         self.new_count -= 1
+        if self.new_count < 0:
+            self.new_count = 0
 
 
     def inc_waiting_count(self):
@@ -852,6 +971,8 @@ class GenericContainer(GenericMedia):
     def dec_waiting_count(self):
 
         self.waiting_count -= 1
+        if self.waiting_count < 0:
+            self.waiting_count = 0
 
 
     def add_slave_dbid(self, dbid):
@@ -1072,7 +1193,7 @@ class GenericRemoteContainer(GenericContainer):
     # Public class methods
 
 
-    def add_child(self, child_obj, no_sort_flag=False):
+    def add_child(self, app_obj, child_obj, no_sort_flag=False):
 
         """Can be called by anything.
 
@@ -1080,6 +1201,8 @@ class GenericRemoteContainer(GenericContainer):
 
         Args:
 
+            app_obj (mainapp.TartubeApp): The main application
+
             child_obj (media.Video): The child object
 
             no_sort_flag (bool): True when the calling code wants to delay
@@ -1094,123 +1217,39 @@ class GenericRemoteContainer(GenericContainer):
 
             self.child_list.append(child_obj)
             if not no_sort_flag:
-                self.sort_children()
+                self.sort_children(app_obj)
 
             if isinstance(child_obj, Video):
                 self.vid_count += 1
 
 
-    def do_sort(self, obj1, obj2):
-
-        """Sorting function used by functools.cmp_to_key(), and called by
-        self.sort_children().
-
-        Sort videos by upload time, with the most recent video first.
-
-        When downloading a channel or playlist, we assume that YouTube (etc)
-        supplies us with the most recent upload first. Therefore, when the
-        upload time is the same, sort by the order in youtube-dl fetches the
-        videos.
-
-        Args:
-
-            obj1, obj2 (media.Video) - Video objects being sorted
-
-        Returns:
-
-            -1 if obj1 comes first, 1 if obj2 comes first, 0 if they are equal
-
-        """
-
-        # Livestreams come before everything else
-        if obj1.live_mode > obj2.live_mode:
-            return -1
-        elif obj1.live_mode < obj2.live_mode:
-            return 1
-
-        # The video's index is not relevant unless sorting a playlist
-        elif isinstance(self, Playlist) \
-        and obj1.index is not None and obj2.index is not None:
-            if obj1.index < obj2.index:
-                return -1
-            else:
-                return 1
-
-        # Otherwise sort by upload time
-        elif obj1.upload_time is not None and obj2.upload_time is not None:
-            if obj1.upload_time > obj2.upload_time:
-                return -1
-            elif obj1.upload_time < obj2.upload_time:
-                return 1
-            elif obj1.receive_time is not None \
-            and obj2.receive_time is not None:
-                if obj1.receive_time < obj2.receive_time:
-                    return -1
-                elif obj1.receive_time > obj2.receive_time:
-                    return 1
-                else:
-                    return 0
-        else:
-            return 0
-
-
-    def sort_children(self):
+    def sort_children(self, app_obj):
 
         """Can be called by anything. For example, called by self.add_child().
 
-        Sorts the child media.Video objects by upload time.
+        Sorts the child media.Video objects using the standard video-sorting
+        function.
+
+        Args:
+
+            app_obj (mainapp.TartubeApp): The main application
+
         """
 
+        self.last_sort_mode = app_obj.catalogue_sort_mode
+
         # Sort a copy of the list to prevent 'list modified during sort'
         #   errors
         while True:
 
             copy_list = self.child_list.copy()
-            copy_list.sort(key=functools.cmp_to_key(self.do_sort))
+            copy_list.sort(key=functools.cmp_to_key(app_obj.video_compare))
 
             if len(copy_list) == len(self.child_list):
                 self.child_list = copy_list.copy()
                 break
 
 
-        self.child_list.sort(key=functools.cmp_to_key(self.do_sort))
-
-
-    def get_livestreams(self, app_obj, live_mode=None):
-
-        """Can be called by anything.
-
-        Returns a list of child media.Video objects which are marked as
-        livestreams.
-
-        If a live_mode is specified, returns either waiting livestreams or
-        broadcasting livestreams (only).
-
-        Args:
-
-            app_obj (mainapp.TartubeApp): The main application
-
-            live_mode (int or None): 1 to return waiting livestreams only, 2 to
-                return broadcasting livestreams only, None to return all
-                livestreams
-
-        Returns:
-
-            A list of media.Video objects (may be an empty list)
-
-        """
-
-        # Check the mainapp.TartubeApp IV is probably cheaper than checking
-        #   self.child_list, as the latter might contain thousands of videos
-        return_list = []
-
-        for video_obj in app_obj.media_reg_live_dict.values():
-            if live_mode is None or video_obj.live_mode == live_mode:
-                return_list.append(video_obj)
-
-        return return_list
-
-
     # Set accessors
 
 
@@ -1291,6 +1330,9 @@ class Video(GenericMedia):
 
     Args:
 
+        app_obj (mainapp.TartubeApp): The main application (not stored as an
+            IV)
+
         dbid (int): A unique ID for this media data object
 
         name (str): The video name
@@ -1310,7 +1352,7 @@ class Video(GenericMedia):
     # Standard class methods
 
 
-    def __init__(self, dbid, name, parent_obj=None, options_obj=None,
+    def __init__(self, app_obj, dbid, name, parent_obj=None, options_obj=None,
     no_sort_flag=False, dummy_flag=False):
 
         # IV list - class objects
@@ -1353,18 +1395,35 @@ class Video(GenericMedia):
         #   video, or False if the downloads.DownloadManager object should
         #   decide whether to simulate, or not
         self.dl_sim_flag = False
+
         # Livestream mode: 0 if the video is not a livestream (or if it was a
         #   livestream which has now finished, and behaves like a normal
         #   uploaded video), 1 if the livestream has not started, 2 if the
         #   livestream is currently being broadcast
         # (Using a numerical mode makes the sorting algorithms more efficient)
         self.live_mode = 0
+        # YouTube 'premiere' videos and actual livestreams are treated in the
+        #   same way, except that for the former, this flag is set to True
+        #   (and a different background colour is used in the Video Catalogue)
+        self.live_debut_flag = False
         # Flag set to True for a video which was a livestream (self.live_mode
         #   = 1 or 2), but is now not (self.live_mode = 0). Once a livestream
         #   video has been marked as a normal video, it can't be marked as a
         #   livestream again. (This prevents any problems in reading the RSS
         #   feeds from continually marking an old video as a livestream again)
         self.was_live_flag = False
+        # The time (matches time.time()) at which a livestream is due to start.
+        #   YouTube supplies an approximate time (perhaps in hours or days),
+        #   but it's still useful for sorting
+        # (For the benefit of the sorting functions, the default value is 0)
+        self.live_time = 0
+        # When YouTube provides a livestream message,
+        #   utils.extract_livestream_data() converts it into some text that can
+        #   be displayed in the Video Catalogue
+        # The text to display. If the video is not a livestream, or has already
+        #   started, or has already finished, or no recognised message was
+        #   provided, an empty string
+        self.live_msg = ''
 
         # Flag set to True if the video is archived, meaning that it can't be
         #   auto-deleted (but it can still be deleted manually by the user)
@@ -1446,25 +1505,23 @@ class Video(GenericMedia):
 
         # IVs used only when the download operation is launched from the
         #   Classic Mode Tab
-        # To save database loading time, these IVs are only added when needed
-        #   (via a call to self.set_dummy() )
         # Flag set to True if this is a dummy media.Video object
         self.dummy_flag = False
-#        # The destination directory for the download
-#        self.dummy_dir = None
-#        # The full path to a downloaded file, if available
-#        self.dummy_path = None
-#        # The video/audio format to use; must be one of the strings in
-#        #   formats.VIDEO_FORMAT_LIST or formats.AUDIO_FORMAT_LIST (or None to
-#        #   use the format(s) specified by the General Options Manager)
-#        self.dummy_format = None
+        # The destination directory for the download
+        self.dummy_dir = None
+        # The full path to a downloaded file, if available
+        self.dummy_path = None
+        # The video/audio format to use; must be one of the strings in
+        #   formats.VIDEO_FORMAT_LIST or formats.AUDIO_FORMAT_LIST (or None to
+        #   use the format(s) specified by the General Options Manager)
+        self.dummy_format = None
 
         # Code
         # ----
 
         # Update the parent
         if parent_obj:
-            self.parent_obj.add_child(self, no_sort_flag)
+            self.parent_obj.add_child(app_obj, self, no_sort_flag)
 
 
     # Public class methods
@@ -1523,9 +1580,9 @@ class Video(GenericMedia):
             )
 
             if self.live_mode == 1:
-                live_str = '<' + _('WAITING') + '>'
+                live_str = ' <' + _('WAITING') + '>'
             elif self.live_mode == 2:
-                live_str = '<' + _('LIVE') + '>'
+                live_str = ' <' + _('LIVE') + '>'
             else:
                 live_str = ''
 
@@ -1569,7 +1626,7 @@ class Video(GenericMedia):
             else:
                 text += self.source
 
-            if self.dummy_path:
+            if self.dummy_path is not None:
                 text += '\n\n' + _('File:') + '\n' + self.dummy_path
 
         # Apply a maximum line length, if required
@@ -1705,6 +1762,14 @@ class Video(GenericMedia):
         self.file_ext = extension
 
 
+    def set_file_from_path(self, path):
+
+        directory, this_file = os.path.split(path)
+        filename, extension = os.path.splitext(this_file)
+        self.file_name = filename
+        self.file_ext = extension
+
+
     def set_file_size(self, size=None):
 
         self.file_size = size
@@ -1723,6 +1788,26 @@ class Video(GenericMedia):
         self.live_mode = mode
 
 
+    def set_live_data(self, live_data_dict):
+
+        """Interprets the dictionary returned by
+        utils.extract_livestream_data().
+        """
+
+        if 'live_msg' in live_data_dict:
+            self.live_msg = live_data_dict['live_msg']
+        else:
+            self.live_msg = ''
+
+        if 'live_time' in live_data_dict:
+            self.live_time = live_data_dict['live_time']
+        else:
+            self.live_time = 0
+
+        if 'live_debut_flag' in live_data_dict:
+            self.live_debut_flag = live_data_dict['live_debut_flag']
+
+
     def set_missing_flag(self, flag):
 
         if flag:
@@ -2163,24 +2248,47 @@ class Video(GenericMedia):
             return None
 
 
-    def get_receive_date_string(self):
+    def get_receive_date_string(self, pretty_flag=False):
 
         """Can be called by anything.
 
         A modified version of self.get_receive_time_string(), returning just
         the date, not the date and the time.
 
+        Args:
+
+            pretty_flag (bool): If True, the strings 'Today' and 'Yesterday'
+                are returned, when possible
+
         Returns:
 
             The formatted string, or None if self.receive_time is not set
 
         """
 
-        if self.receive_time:
+        if not self.receive_time:
+            return None
+
+        elif not pretty_flag:
             timestamp = datetime.datetime.fromtimestamp(self.receive_time)
             return timestamp.strftime('%Y-%m-%d')
+
         else:
-            return None
+            today = datetime.date.today()
+            today_str = today.strftime('%y%m%d')
+
+            yesterday = datetime.date.today() - datetime.timedelta(days=1)
+            yesterday_str = yesterday.strftime('%y%m%d')
+
+            testday = datetime.datetime.fromtimestamp(self.receive_time)
+            testday_str = testday.strftime('%y%m%d')
+
+            if testday_str == today_str:
+                return _('Today')
+            elif testday_str == yesterday_str:
+                return _('Yesterday')
+            else:
+                return testday.strftime('%Y-%m-%d')
 
 
     def get_receive_time_string(self):
@@ -2318,6 +2426,12 @@ class Channel(GenericRemoteContainer):
         #   operation. For channels on other websites, can be set manually
         self.rss = None
 
+        # The value of mainapp.TartubeApp.catalogue_sort_mode, the last time
+        #   self.sort_children() was called. (When the user changes the sort
+        #   mode, we don't sort self.child_list() until we actually need to
+        #   display it)
+        self.last_sort_mode = 'default'
+
         # Alternative download destination - the dbid of a channel, playlist or
         #   folder in whose directory videos, thumbnails (etc) are downloaded.
         #   By default, set to the dbid of this channel; but can be set to the
@@ -2378,7 +2492,7 @@ class Channel(GenericRemoteContainer):
 
         # Update the parent (if any)
         if self.parent_obj:
-            self.parent_obj.add_child(self)
+            self.parent_obj.add_child(app_obj, self)
 
 
     # Public class methods
@@ -2390,9 +2504,6 @@ class Channel(GenericRemoteContainer):
 #   def del_child():                # Inherited from GenericContainer
 
 
-#   def do_sort():                  # Inherited from GenericRemoteContainer
-
-
 #   def sort_children():            # Inherited from GenericRemoteContainer
 
 
@@ -2490,6 +2601,12 @@ class Playlist(GenericRemoteContainer):
         #   manually by the user for other websites
         self.rss = None
 
+        # The value of mainapp.TartubeApp.catalogue_sort_mode, the last time
+        #   self.sort_children() was called. (When the user changes the sort
+        #   mode, we don't sort self.child_list() until we actually need to
+        #   display it)
+        self.last_sort_mode = 'default'
+
         # Alternative download destination - the dbid of a channel, playlist or
         #   folder in whose directory videos, thumbnails (etc) are downloaded.
         #   By default, set to the dbid of this playlist; but can be set to the
@@ -2550,7 +2667,7 @@ class Playlist(GenericRemoteContainer):
 
         # Update the parent (if any)
         if self.parent_obj:
-            self.parent_obj.add_child(self)
+            self.parent_obj.add_child(app_obj, self)
 
 
     # Public class methods
@@ -2562,9 +2679,6 @@ class Playlist(GenericRemoteContainer):
 #   def del_child():                # Inherited from GenericContainer
 
 
-#   def do_sort():                  # Inherited from GenericRemoteContainer
-
-
 #   def sort_children():            # Inherited from GenericRemoteContainer
 
 
@@ -2672,6 +2786,12 @@ class Folder(GenericContainer):
         #   folder can't be changed
         self.nickname = name
 
+        # The value of mainapp.TartubeApp.catalogue_sort_mode, the last time
+        #   self.sort_children() was called. (When the user changes the sort
+        #   mode, we don't sort self.child_list() until we actually need to
+        #   display it)
+        self.last_sort_mode = 'default'
+
         # Alternative download destination - the dbid of a channel, playlist or
         #   folder in whose directory videos, thumbnails (etc) are downloaded.
         #   By default, set to the dbid of this folder; but can be set to the
@@ -2743,13 +2863,13 @@ class Folder(GenericContainer):
 
         # Update the parent (if any)
         if self.parent_obj:
-            self.parent_obj.add_child(self)
+            self.parent_obj.add_child(app_obj, self)
 
 
     # Public class methods
 
 
-    def add_child(self, child_obj, no_sort_flag=False):
+    def add_child(self, app_obj, child_obj, no_sort_flag=False):
 
         """Can be called by anything.
 
@@ -2758,6 +2878,8 @@ class Folder(GenericContainer):
 
         Args:
 
+            app_obj (mainapp.TartubeApp): The main application
+
             child_obj (media.Video, media.Channel, media.Playlist,
                 media.Folder): The child object
 
@@ -2771,7 +2893,7 @@ class Folder(GenericContainer):
 
             self.child_list.append(child_obj)
             if not no_sort_flag:
-                self.sort_children()
+                self.sort_children(app_obj)
 
             if isinstance(child_obj, Video):
                 self.vid_count += 1
@@ -2810,100 +2932,45 @@ class Folder(GenericContainer):
         return False
 
 
-#   def del_child():                # Inherited from GenericContainer
+    def check_duplicate_video_by_path(self, app_obj, path):
 
+        """Called by mainwin.MainWin.on_window_drag_data_received().
 
-    def do_sort(self, obj1, obj2):
-
-        """Sorting function used by functools.cmp_to_key(), and called by
-        self.sort_children().
-
-        Sorts the child media.Video, media.Channel, media.Playlist and
-        media.Folder objects.
-
-        Firstly, sort by class - folders, channels/playlists, then videos.
-
-        Within folders, channels and playlists, sort alphabetically. Within
-        videos, sort by upload time.
+        A modified version of self.check_duplicate_video(), which checks for
+        media.Video objects with duplicate paths, instead of dupliate URLs.
 
         Args:
 
-            obj1, obj2 (media.Video, media.Channel, media.Playlist or
-                media.Folder) - Media data objects being sorted
+            app_obj (mainapp.TartubeApp): The main application
+
+            path (str): The full file path to check
 
         Returns:
 
-            -1 if obj1 comes first, 1 if obj2 comes first, 0 if they are equal
+            True if any of the child media.Video objects in this folder have
+                the same source URL; False otherwise
 
         """
 
-        if str(obj1.__class__) == str(obj2.__class__) \
-        or (
-            isinstance(obj1, GenericRemoteContainer) \
-            and isinstance(obj2, GenericRemoteContainer)
-        ):
-            if isinstance(obj1, Video):
+        for child_obj in self.child_list:
 
-                # Livestreams come before everything else
-                if obj1.live_mode > obj2.live_mode:
-                    return -1
-                elif obj1.live_mode < obj2.live_mode:
-                    return 1
+            if isinstance(child_obj, Video) \
+            and child_obj.file_name is not None:
 
-                # Otherwise sort by upload time, then by receive time
-                elif obj1.upload_time is not None \
-                and obj2.upload_time is not None:
-                    if obj1.upload_time > obj2.upload_time:
-                        return -1
-                    elif obj1.upload_time < obj2.upload_time:
-                        return 1
-                    elif obj1.receive_time is not None \
-                    and obj2.receive_time is not None:
-                        # In private folders (e.g. 'All Videos'), the most
-                        #   recently received video goes to the top of the list
-                        if self.priv_flag:
-                            if obj1.receive_time > obj2.receive_time:
-                                return -1
-                            elif obj1.receive_time < obj2.receive_time:
-                                return 1
-                            else:
-                                return 0
-                        # ...but for everything else, the sorting algorithm is
-                        #   the same as for GenericRemoteContainer.do_sort(),
-                        #   in which we assume the website is sending us
-                        #   videos, newest first
-                        else:
-                            if obj1.receive_time < obj2.receive_time:
-                                return -1
-                            elif obj1.receive_time > obj2.receive_time:
-                                return 1
-                            else:
-                                return 0
-                    else:
-                        return 0
-                else:
-                    return 0
-            else:
-                if obj1.name.lower() < obj2.name.lower():
-                    return -1
-                elif obj1.name.lower() > obj2.name.lower():
-                    return 1
-                else:
-                    return 0
-        else:
-            if isinstance(obj1, Folder):
-                return -1
-            elif isinstance(obj2, Folder):
-                return 1
-            elif isinstance(obj1, Channel) or isinstance(obj1, Playlist):
-                return -1
-            elif isinstance(obj2, Channel) or isinstance(obj2, Playlist):
-                return 1
-            else:
-                return 0
+                child_path = child_obj.get_actual_path(app_obj)
+                if child_path is not None and child_path == path:
+
+                    # Duplicate found
+                    return True
+
+        # No duplicate found
+        return False
 
 
-    def sort_children(self):
+#   def del_child():                # Inherited from GenericContainer
+
+
+    def sort_children(self, app_obj):
 
         """Can be called by anything. For example, called by self.add_child().
 
@@ -2911,15 +2978,16 @@ class Folder(GenericContainer):
         media.Folder objects.
         """
 
-        # v1.0.002: At the end of a download operation, I am seeing 'list
-        #   modified during sort' errors. Not sure what the cause is, but we
-        #   can prevent it by sorting a copy of the list, rather than the list
-        #   itself. If the list itself is modified during the sort, sort it
-        #   again
+        self.last_sort_mode = app_obj.catalogue_sort_mode
+
+        # Sort a copy of the list to prevent 'list modified during sort'
+        #   errors
         while True:
 
             copy_list = self.child_list.copy()
-            copy_list.sort(key=functools.cmp_to_key(self.do_sort))
+            copy_list.sort(
+                key=functools.cmp_to_key(app_obj.folder_child_compare),
+            )
 
             if len(copy_list) == len(self.child_list):
                 self.child_list = copy_list.copy()
@@ -2967,3 +3035,117 @@ class Folder(GenericContainer):
         collapse neatly in my IDE."""
 
         pass
+
+
+class Scheduled(object):
+
+    """Python class that handles a scheduled download operation.
+
+    Args:
+
+        name (str): Unique name for the scheduled download (a string, minimum 1
+            character)
+
+        dl_mode (str): Download operation type: 'sim' (for simulated
+            downloads), 'real' (for real downloads) or 'custom' (for custom
+            downloads)
+
+        start_mode (str) 'none' to disable this schedule, 'start' to perform
+            the operation whenever Tartube starts, or 'scheduled' to perform
+            the operation at regular intervals
+
+    """
+
+
+    # Standard class methods
+
+
+    def __init__(self, name, dl_mode, start_mode):
+
+        # IV list - other
+        # ---------------
+        # Unique name for the scheduled download (a string, minimum 1
+        #   character)
+        self.name = name
+
+        # Download operation type: 'sim' (for simulated downloads), 'real' (for
+        #   real downloads) or 'custom' (for custom downloads)
+        self.dl_mode = dl_mode
+        # Start mode - 'none' to disable this schedule, 'start' to perform the
+        #   operation whenever Tartube starts, or 'scheduled' to perform the
+        #   operation at regular intervals
+        self.start_mode = start_mode
+
+        # The time between scheduled downloads, when self.start_mode is
+        #   'scheduled' (minimum value 1, ignored for other values of
+        #   self.start_mode)
+        self.wait_value = 2
+        # ...using this unit (any of the values in formats.TIME_METRIC_LIST;
+        #   the 'seconds' value is not available in the edit window's combobox)
+        self.wait_unit = 'hours'
+
+        # The time (system time, in seconds) at which this scheduled download
+        #   last started (regardless of whether it was scheduled to begin at
+        #   that time, or not)
+        self.last_time = 0
+        # When self.start_mode is 'start', mainapp.TartubeApp.start sets this
+        #   value to the time at which the scheduled download should start
+        #   (which will be a few seconds after startup)
+        # Once the scheduled download is started, the value is set back to 0
+        self.only_time = 0
+
+        # When multiple scheduled downloads are due to start at the same time,
+        #   a flag that marks this as an exclusive scheduled download
+        # Scheduled downloads are checked in the order specified by
+        #   mainapp.TartubeApp.scheduled_list. If this is flag is True for any
+        #   of them, only one of the flagged downloads starts
+        self.exclusive_flag = False
+        # When this scheduled download is due to start, what to do if another
+        #   download operation is in progress: 'join' to add media data objects
+        #   to the current download operation (at the end of the existing
+        #   list), 'priority' to add media data objects to the current download
+        #   operation (at the beginning of the existing list), 'skip' to wait
+        #   until the next scheduled operation time instead
+        # Note that this affects a download operation already in progress,
+        #   not one which is about to start (because multiple scheduled
+        #   downloads are due to start at the same time)
+        self.join_mode = 'skip'
+
+        # Flag set to True if Tartube should shut down after this scheduled
+        #   download operation occurs, False if not
+        self.shutdown_flag = False
+        # Flag set to True if the whole channel/playlist/folder should be
+        #   checked/downloaded, regardless of the value of
+        #   mainapp.TartubeApp.operation_limit_flag, etc
+        self.ignore_limits_flag = False
+
+        # Flag set to True if the download operation should encompass all
+        #   media data objects
+        self.all_flag = True
+        # List of names of media.Channel, media.Playlist and media.Folder
+        #   objects to add to the download operation (not the objects
+        #   themselves). All of their children are also added). Ignored if
+        #   self.all_flag is True
+        self.media_list = []
+
+
+    # Public class methods
+
+
+    # Set accessors
+
+
+    def set_last_time(self, time):
+
+        self.last_time = time
+
+
+    def add_media(self, name):
+
+        self.media_list.append(name)
+        self.all_flag = False
+
+
+    def set_only_time(self, time):
+
+        self.only_time = time
