@@ -40,9 +40,7 @@ import utils
 
 class OptionsManager(object):
 
-    """Called by mainapp.TartubeApp.OptionsManager().
-
-    Partially based on the OptionsManager class in youtube-dl-gui.
+    """Partially based on the OptionsManager class in youtube-dl-gui.
 
     This class handles settings for downloading media. Unlike youtube-dl-gui,
     which has one group of download options applied to all downloads, this
@@ -206,6 +204,15 @@ class OptionsManager(object):
         referer (str): Specify a custom referer to use if the video access is
             restricted to one domain
 
+        min_sleep_interval (int): Number of seconds to sleep before each
+            download when used alone, or a lower bound of a range for
+            randomised sleep before each download (minimum possible number of
+            seconds to sleep) when used along with max_sleep_interval
+
+        max_sleep_interval (int): Upper bound of a range for randomized sleep
+            before each download (maximum possible number of seconds to sleep).
+            Can only be used along with a non-zero value for min_sleep_interval
+
     VIDEO FORMAT OPTIONS
 
         video_format (str): Video format to download. When this option is set
@@ -328,7 +335,7 @@ class OptionsManager(object):
 
         [in youtube-dl-gui, this was named 'cmd_args']
 
-        extra_cmd_string: String that contains extra youtube-dl options
+        extra_cmd_string (str): String that contains extra youtube-dl options
             separated by spaces. Components containing whitespace can be
             enclosed within double quotes "..."
 
@@ -416,10 +423,22 @@ class OptionsManager(object):
     # Standard class methods
 
 
-    def __init__(self):
+    def __init__(self, uid, name, dbid=None):
 
         # IV list - other
         # ---------------
+        # Unique ID for this options manager
+        self.uid = uid
+        # A non-unique name for this options manager. Managers that are
+        #   attached to a media data object have the same name as that object.
+        #   (The name is not unique because, for example, videos could have the
+        #   same name as a channel; it's up to the user to avoid duplicate
+        #   names)
+        self.name = name
+        # If this object is attached to a media data object, the .dbid of that
+        #   object; otherwise None
+        self.dbid = dbid
+
         # Dictionary of download options for youtube-dl, set by a call to
         #   self.reset_options
         self.options_dict = {}
@@ -438,9 +457,9 @@ class OptionsManager(object):
     def clone_options(self, other_options_manager_obj):
 
         """Called by mainapp.TartubeApp.apply_download_options() and
-        .clone_general_options_manager().
+        .clone_general_download_options().
 
-        Clones download options from the specified object into those object,
+        Clones download options from the specified object into this object,
         completely replacing this object's download options.
 
         Args:
@@ -559,6 +578,8 @@ class OptionsManager(object):
             'prefer_insecure': False,
             'user_agent': '',
             'referer': '',
+            'min_sleep_interval': 0,
+            'max_sleep_interval': 0,
             # VIDEO FORMAT OPTIONS
             'video_format': '0',
             'all_formats': False,
@@ -622,7 +643,7 @@ class OptionsManager(object):
 
     def set_classic_mode_options(self):
 
-        """Called by mainapp.TartubeApp.apply_classic_downoad_options().
+        """Called by mainapp.TartubeApp.apply_classic_download_options().
 
         When the user applies download options in the Classic Mode Tab, a few
         options should have different default values; this function sets them.
@@ -649,6 +670,14 @@ class OptionsManager(object):
         self.options_dict['sim_keep_thumbnail'] = False
 
 
+    # Set accessors
+
+
+    def set_dbid(self, dbid):
+
+        self.dbid = dbid
+
+
 class OptionsParser(object):
 
     """Called by downloads.DownloadManager.__init__() and by
@@ -785,6 +814,10 @@ class OptionsParser(object):
             OptionHolder('user_agent', '--user-agent', ''),
             # --referer URL
             OptionHolder('referer', '--referer', ''),
+            # --sleep-interval SECONDS
+            OptionHolder('min_sleep_interval', '--sleep-interval', 0),
+            # --max-sleep-interval SECONDS
+            OptionHolder('max_sleep_interval', '--max-sleep-interval', 0),
             # VIDEO FORMAT OPTIONS
             # -f, --format FORMAT
             OptionHolder('video_format', '-f', '0'),
@@ -913,8 +946,9 @@ class OptionsParser(object):
             options_manager_obj (options.OptionsManager): The object containing
                 the download options for this media data object
 
-            operation_type (str): 'sim', 'real', 'custom', 'classic' (matching
-                possible values of downloads.DownloadManager.operation_type)
+            operation_type (str): 'sim', 'real', 'custom', 'classic_sim',
+                'classic_real', 'classic_custom' (matching possible values of
+                downloads.DownloadManager.operation_type)
 
         Returns:
 
@@ -1009,7 +1043,7 @@ class OptionsParser(object):
         # Parse the 'extra_cmd_string' option, which can contain arguments
         #   inside double quotes "..." (arguments that can therefore contain
         #   whitespace)
-        parsed_list = utils.parse_ytdl_options(copy_dict['extra_cmd_string'])
+        parsed_list = utils.parse_options(copy_dict['extra_cmd_string'])
         for item in parsed_list:
             options_list.append(item)
 
@@ -1089,14 +1123,17 @@ class OptionsParser(object):
 
             copy_dict (dict): Copy of the original options dictionary
 
-            operation_type (str): 'sim', 'real', 'custom', 'classic' (matching
-                possible values of downloads.DownloadManager.operation_type)
+            operation_type (str): 'sim', 'real', 'custom', 'classic_sim',
+                'classic_real', 'classic_custom' (matching possible values of
+                downloads.DownloadManager.operation_type)
 
         """
 
         # Special case: if a download operation was launched from the Classic
         #   Mode Tab, the save path is specified in that tab
-        if operation_type == 'classic':
+        if operation_type == 'classic_sim' \
+        or operation_type == 'classic_real' \
+        or operation_type == 'classic_custom':
 
             save_path = media_data_obj.dummy_dir
 
@@ -1158,8 +1195,11 @@ class OptionsParser(object):
 
             # Special case: if a download operation was launched from the
             #   Classic Mode Tab, the video format may be specified by that tab
-            if operation_type == 'classic' \
-            and media_data_obj.dummy_format:
+            if (
+                operation_type == 'classic_sim' \
+                or operation_type == 'classic_real' \
+                or operation_type == 'classic_custom'
+            ) and media_data_obj.dummy_format:
 
                 dummy_format = media_data_obj.dummy_format
 
@@ -1208,7 +1248,7 @@ class OptionsParser(object):
         #   formats; if the format isn't available for some videos, we'll get
         #   an error for each of them (rather than the simulated download we
         #   were hoping for)
-        if operation_type == 'sim':
+        if operation_type == 'sim' or operation_type == 'classic_sim':
 
             copy_dict['video_format'] = '0'
             copy_dict['all_formats'] = False
